#include"usercode.h"
#include "stm32f4xx.h"
#include "stm32f4xx_can.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_exti.h"
#include "stm32f4xx_syscfg.h"
#include "stm32f4xx_rcc.h"
#include "misc.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "interrupt.h"
#include "stm32f4xx_tim.h"

#define SWITCH(X) (X
#define CHECK(X) X)
#define CRC_TEST SWITCH (0)
#define CHECKSUM_TEST SWITCH (1)

extern xQueueHandle xQueueCAN1;
uint8_t array_receive[2000];

#if CHECK(CRC_TEST)
uint32_t checkCRC = 0;
uint32_t CRC_in = 0;
uint32_t Calc_CRC(uint8_t *data, uint16_t arrayLength);
uint8_t Check_REC_CRC(uint32_t fu, uint32_t bar);
#endif

#if CHECK(CHECKSUM_TEST)
uint32_t checkSum = 0;
uint32_t inCheckSum = 0;
uint32_t calculateChecksum(uint8_t *data, uint16_t arrayLength);
uint8_t checkCheckSums(uint32_t foo, uint32_t bar);
#endif
/**
  * @brief  Wenn CAN1 etwas empf�ngt wird die CAN-Frame �ber die Queue direkt �bergeben.
  * 		Diese Task baut aus den Frames das gesendete Array wieder auf.
  * @param  pvParameters not used
  * @retval None
  */
void ArrayReceive(void * pvParameters)
{
	static volatile uint16_t i=0,j=0;
	static uint8_t rec_state=0;
	CanRxMsg RxMessage;
	uint8_t is_Error;


	for( ;; )
	{

		/*wait till a Message was received*/
		xQueueReceive( xQueueCAN1, &( RxMessage ),portMAX_DELAY  );

		/*was it a start, data or end Id otherwise do not care*/
		switch(RxMessage.StdId)
		{
			/*a start Id*/
			case 0x100:
				/*we are in reset state*/
				if(rec_state==0)
				{
					rec_state=1;	//ready to write something
					i=0;
				}
				else
				{
					rec_state=0;	//error we were already in write data state
					i=0;
				}

			break;

			//

			/*a end Id*/
			case 0x102:
				/*we are in data write state */
				if(rec_state==1)
				{
					rec_state=0;
					i=0;
					#if CHECK(CRC_TEST)
					checkCRC = 0;
					CRC_in = 0;
					CRC_in = CRC_in + (RxMessage.Data[0]<<24);
					CRC_in = CRC_in + (RxMessage.Data[1]<<16);
					CRC_in = CRC_in + (RxMessage.Data[2]<<8);
					CRC_in = CRC_in + RxMessage.Data[3];

					checkCRC=Calc_CRC(array_receive,2000);

					is_Error=Check_REC_CRC(CRC_in,checkCRC);
					#endif

					#if CHECK(CHECKSUM_TEST)
					inCheckSum = 0;
					inCheckSum = inCheckSum + (RxMessage.Data[0]<<24);
					inCheckSum = inCheckSum + (RxMessage.Data[1]<<16);
					inCheckSum = inCheckSum + (RxMessage.Data[2]<<8);
					inCheckSum = inCheckSum + RxMessage.Data[3];

					checkSum = calculateChecksum(array_receive,2000);

					is_Error=checkCheckSums(checkSum,inCheckSum);
					#endif

					if(!is_Error)
					{
						GPIO_SetBits(GPIOD, GPIO_Pin_12);
						/*Wait for 250ms since LastWakeTime*/
						vTaskDelay(250);
						GPIO_ResetBits(GPIOD, GPIO_Pin_12);
					}
					else
					{
						GPIO_SetBits(GPIOD, GPIO_Pin_14);
						/*Wait for 250ms since LastWakeTime*/
						vTaskDelay(250);
						GPIO_ResetBits(GPIOD, GPIO_Pin_14);
					}
				}
			break;

			/*everything else do not care*/
			default:
				break;

		}
	}

}

#if CHECK(CRC_TEST)
/**
  * @brief  calculates the CRC (used for checking of received frames)
  * @param  the data to check
  * @retval the CRC value (0 if the received CRC is correct)
  */
uint32_t Calc_CRC(uint8_t *data, uint16_t arrayLength)
{
	uint32_t CRC_Check=0;
	uint32_t CRC_Proceed=0;
	uint32_t* CRC_pBuff;
	CRC_ResetDR();
	if((arrayLength%4)==0)
	{
		CRC_pBuff = (uint32_t*)data;
		CRC_Proceed=arrayLength>>2;
		CRC_Check=CRC_CalcBlockCRC(CRC_pBuff, CRC_Proceed);
	}
	else
	{
		while(1)
		{
			GPIO_SetBits(GPIOD, GPIO_Pin_14);
		}
	}
	return (CRC_Check);
}
/**
  * @brief  calculates the CRC (used for checking of received frames)
  * @param  the data to check
  * @retval the CRC value (0 if the received CRC is correct)
  */
uint8_t Check_REC_CRC(uint32_t fu, uint32_t bar)
{
	uint8_t isError=0;
	if(fu == bar)
	{
		isError=0;
	}
	else
	{
		isError=1;
	}
	return(isError);
}
#endif

#if CHECK(CHECKSUM_TEST)
uint32_t calculateChecksum(uint8_t *data, uint16_t arrayLength)
{
	uint16_t index;
	uint32_t checkSum=0;
	for(index=0; index<arrayLength; index++)
	{
		checkSum+=data[index];
	}
	return(checkSum);
}

uint8_t checkCheckSums(uint32_t foo, uint32_t bar)
{
	uint8_t isError=0;
	if(foo==bar)
	{
		isError=0;
	}
	else
	{
		isError=1;
	}
	return(isError);
}
#endif
